<?php

abstract class APP_Stat{

	const HOUR = 'hour';
	const DAY = 'day';
	const WEEK = 'week';
	const MONTH = 'month';
	const YEAR = 'year';

	/**
	 * Take a set of timestamps and condense into more general timestamps
	 * By default includes counts of how many timestamps were compressed together
	 * @param  array   $dates          An array of timestamps
	 * @param  string  $compression    Compression level. Can be APP_Stats:HOUR, APP_Stats:Day, APP_Stats:Week
	 *                                 	APP_Stats:MONTH, or APP_Stats:YEAR
	 * @param  boolean $include_counts Whether to include counts in the array
	 * @return array                   If $include_counts is set to true, returns an assoc array with dates 
	 *                                    as the keys, and counts as the values. Otherwise just returs an 
	 *                                    array of dates
	 */
	protected function compress( $dates, $compression = self::HOUR, $include_counts = true ){

		$counts = array();
		foreach( $dates as $date ){
			$counts[] = $this->abbrvTime( strtotime( $date ), $compression );
		}

		return ( $include_counts ) ? array_count_values( $counts ) : array_unique( $counts );
	}

	/**
	 * Take a set of timestamp counts and condense into more general timestamp counts
	 * @param  array   $dates          An array of timestamps and counts
	 * @param  string  $compression    Compression level. Can be APP_Stats:HOUR, APP_Stats:Day, APP_Stats:Week
	 *                                 	APP_Stats:MONTH, or APP_Stats:YEAR
	 * @param  boolean $include_counts Whether to include counts in the array
	 * @return array                   If $include_counts is set to true, returns an assoc array with dates 
	 *                                    as the keys, and counts as the values. Otherwise just returs an 
	 *                                    array of dates
	 */
	protected function compressCounts( $dates, $compression = self::HOUR ){

		$counts = array();
		foreach( $dates as $date => $count ){
			$new_date = $this->abbrvTime( $date, $compression );
			if( !isset( $counts[ $new_date ] ) ){
				$counts[ $new_date ] = $count;
			}else{
				$counts[ $new_date ] += $count;
			}
		}

		return $counts;

	}

	/**
	 * Expand a set of datestamps with zeros to make it fit the given dates
	 * @param  [type] $dates      [description]
	 * @param  [type] $level      [description]
	 * @param  [type] $start_date [description]
	 * @param  [type] $end_date   [description]
	 * @return [type]             [description]
	 */
	public function expandCounts( $data, $compression = self::HOUR, $start_date, $end_date ){

		$start = date_create( date( 'Y-m-d H:i:s', $this->abbrvTime( $start_date, $compression ) ) );
		$end = date_create( date( 'Y-m-d H:i:s', $this->abbrvTime( $end_date, $compression ) ) );
		switch( $compression ){
			case self::YEAR:
				$interval = new DateInterval("P1Y");
				break;
			case self::MONTH:
				$interval = new DateInterval("P1M");
				break;
			case self::WEEK:
				$interval = new DateInterval("P1W");
				break;
			case self::DAY:
				$interval = new DateInterval("P1D");
				break;
			default:
				$interval = new DateInterval("PT1H");
				break;
		}

		$dates = array();
		$period = new DatePeriod( $start, $interval, $end );
		foreach( $period as $date ){
			$time = $this->abbrvTime( $date->getTimestamp(), $compression );
			$dates[ $time ] = 0;
		}
		$dates[ $this->abbrvTime( $start->getTimestamp(), $compression ) ] = 0;
		$dates[ $this->abbrvTime( $end->getTimestamp(), $compression ) ] = 0;

		foreach( $data as $time => $count ){
			$abbrv = $this->abbrvTime( $time, $compression );
			$dates[ $abbrv ] += $count;
		}

		return $dates;

	}

	protected function abbrvTime( $time, $compression ){

		$date = getdate( $time );

		switch( $compression ){
			case self::YEAR:
				$date['seconds'] = 0;
				$date['minutes'] = 0;
				$date['hours'] = 0;
				$date['mday'] = 1;
				$date['mon'] = 1;
				break;
			case self::MONTH:
				$date['seconds'] = 0;
				$date['minutes'] = 0;
				$date['hours'] = 0;
				$date['mday'] = 1;
				break;
			case self::DAY:
				$date['seconds'] = 0;
				$date['minutes'] = 0;
				$date['hours'] = 0;
				break;
			default:
				$date['seconds'] = 0;
				$date['minutes'] = 0;
				break;
		}

		return mktime( $date['hours'], $date['minutes'], $date['seconds'], $date['mon'], $date['mday'], $date['year'] );

	}

	protected function filterByProperty( $property, $values ){

		if( ! is_array( $values ) ){
			$values = array( $values );
		}

		$items = array();
		foreach( $this->posts as $post ){
			if( ! isset( $post->$property ) )
				continue;

			if( in_array( $post->$property, $values ) )
				$items[] = $post;
		}

		return $this->create( $items );

	}

	protected function retrieveProperty( $property ){
		$values = wp_list_pluck( $this->posts, $property );
		return array_unique( $values );
	}

	protected function countProperty( $property ){
		$values = wp_list_pluck( $this->posts, $property );
		return array_count_values( $values );
	}

	protected function dataAverage( $data ){
		return array_sum( $data ) / count( $data );
	}

	protected function dataMedian( $data ){

		sort( $data );
		$count = count( $data );

		$midpoint = floor( ( $count-1 ) / 2 );
    	if( $count % 2 ) {
        	$median = $data[ $midpoint ];
    	} else {
        	$low = $data[ $midpoint ];
        	$high = $data[ $midpoint + 1 ];
        	$median = ( ( $low + $high ) / 2 );
    	}

    	return $median;
	}

	protected function dataMode( $data ){
		$values = array_count_values( $data );
    	return array_search( max( $values ), $values );
	}

	protected function dataMax( $data ){
		return array(
			'key' => array_search( max( $data ), $data ),
    		'value' => max( $data )
		);
	}

	protected function dataMin( $data ){
		return array(
			'key' => array_search( min( $data ), $data ),
    		'value' => min( $data )
		);
	}

	// Create new instances of subclasses too
	protected function create( $items ){
		$class = get_class( $this );
		return new $class( $items );
	}

}