<?php
/**
 * Wallet One processing gateway. Uses the AppThemes Payment API.
 *
 * @package DKF\WalletOne
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

/**
 * Provides implementation of AppThemes Payment Gateway for use with Wallet One.
 */
class DKF_WalletOne extends APP_Boomerang implements APP_Recurring_Payment_Processor {

	/**
	 * Gateway ID.
	 */
	const ID = 'wallet_one';

	/**
	 * Payment request URL.
	 */
	const POST_URL = 'https://wl.walletone.com/checkout/checkout/Index';

	/**
	 * Recurring Payment request URL.
	 */
	const POST_RECUR_URL = '???';

	const SECRET_KEY = 'SECRET_KEY';

	/**
	 * Online store ID (account number), received at registration.
	 */
	const MERCHANT_ID = 'WMI_MERCHANT_ID';

	/**
	 * Order amount, showing 2 digits after decimal point. Full stop symbol is
	 * used as decimal point. It is compulsory to indicate 2 digits after
	 * decimal.
	 */
	const PAYMENT_AMOUNT = 'WMI_PAYMENT_AMOUNT';

	/**
	 * Currency ID (ISO 4217):
	 *  - 643 — Russian Rubles
	 *  - 710 — South African Rand
	 *  - 840 — US Dollar
	 *  - 978 — Euro
	 *  - 980 — Ukrainian Hryvnia
	 *  - 398 — Kazakhstani tenge
	 *  - 974 — Belarusian rubles
	 *  - 972 — Tajik Somoni
	 */
	const CURRENCY_ID = 'WMI_CURRENCY_ID';

	/**
	 * Order ID in online store’s accounting system. Each order has to have a
	 * unique ID number.
	 */
	const PAYMENT_NO = 'WMI_PAYMENT_NO';

	/**
	 * Order description (List of goods, etc.) is reflected on the order payment
	 * page, as well as in payment history of the buyer maximum amount of
	 * symbols – 255.
	 */
	const DESCRIPTION = 'WMI_DESCRIPTION';

	/**
	 * Online store web-pages addresses (URL), where buyer will be directed
	 * after successful payment.
	 */
	const SUCCESS_URL = 'WMI_SUCCESS_URL';

	/**
	 * Online store web-pages addresses (URL), where buyer will be directed
	 * after successful or unsuccessful payment.
	 */
	const FAIL_URL = 'WMI_FAIL_URL';

	/**
	 * The expiry date of payment. Date is indicated in Western European time
	 * zone (UTC+0) and should be more than current date value(ISO 8601),
	 * for example: 2013-11-09T16:18:52.
	 *
	 * Please note: invoice validity period may not exceed 30 days after the
	 * moment of issue!
	 */
	const EXPIRED_DATE = 'WMI_EXPIRED_DATE';

	/**
	 * These parameters allow you to define available payment methods.
	 */
	const PTENABLED  = 'WMI_PTENABLED';

	/**
	 * These parameters allow you to define unavailable payment methods.
	 */
	const PTDISABLED = 'WMI_PTDISABLED';

	/**
	 * Payer default login. The value of this parameter will be inserted into
	 * login field on authorisation. Possible formats: e-mail, phone number in
	 * international format.
	 */
	const RECIPIENT_LOGIN = 'WMI_RECIPIENT_LOGIN';

	/**
	 * The telephone number of payer in international format.
	 *
	 * For example, +71234567890.
	 *
	 * The meaning of this field will be used for:
	 *  - it will be placed automatically in forms of some methods of payment;
	 *  - it will be used for determination of payer's country;
	 *  - it will be used for search of client's invoices in banks and systems
	 *    of e-commerce.
	 */
	const CUSTOMER_PHONE = 'WMI_CUSTOMER_PHONE';

	/**
	 * Payer’s firstname.
	 *
	 * This value will be inserted into fields of some payment type forms.
	 */
	const CUSTOMER_FIRSTNAME = 'WMI_CUSTOMER_FIRSTNAME';

	/**
	 * Payer’s lastname.
	 *
	 * This value will be inserted into fields of some payment type forms.
	 */
	const CUSTOMER_LASTNAME = 'WMI_CUSTOMER_LASTNAME';

	/**
	 * Payer’s e-mail.
	 *
	 * This value will be inserted into fields of some payment type forms.
	 */
	const CUSTOMER_EMAIL = 'WMI_CUSTOMER_EMAIL';

	/**
	 * Interface language is defined automatically, but you can choose to set it:
	 *  - ru-RU — russian;
	 *  - en-US — english.
	 */
	const CULTURE_ID = 'WMI_CULTURE_ID';

	/**
	 * Allows to show the methods of payment available for user according to his
	 * location.
	 *
	 * 0 — shows the methods regardless of the user's country;
	 * 1 — the user's country and the display of methods determines according to
	 * the IP.
	 */
	const AutoLocation = 'WMI_AutoLocation';

	/**
	 * Payment form signature, formed using online store’s “secret key”.
	 *
	 * Requirement to verify this parameter can be set in “online-store”
	 * settings. Read more in the section «Payment form Protection».
	 */
	const SIGNATURE = 'WMI_SIGNATURE';

	/**
	 * Order state posted by IPN.
	 */
	const ORDER_STATE = 'WMI_ORDER_STATE';

	/**
	 * Submerchant ID (for aggregators).
	 */
	const PSP_MERCHANT_ID = 'WMI_PSP_MERCHANT_ID';

	/**
	 * Creates the Gateway class with the required information to display it.
	 */
	public function __construct() {
		parent::__construct( self::ID, array(
			'admin'     => __( 'Wallet One Checkout', DKF_W1_TD ),
			'dropdown'  => __( 'Wallet One Checkout', DKF_W1_TD ),
			//'recurring' => true,
		) );
	}

	/**
	 * Returns an array representing the form to output for admin configuration.
	 *
	 * @return array scbForms style form array.
	 */
	public function form() {

		return array(
			array(
				'title' => __( 'General', DKF_W1_TD ),
				'fields' => array(
					array(
						'title' => __( 'Merchant ID', DKF_W1_TD ),
						'name'  => self::MERCHANT_ID,
						'type'  => 'text',
						'tip'   => __( 'Online store ID (account number), received at registration.', DKF_W1_TD ),
					),
					array(
						'title' => __( 'Secret Key', DKF_W1_TD ),
						'name'  => self::SECRET_KEY,
						'type'  => 'text',
						'tip'   => __( 'The secret key that you have set in your personal Wallet One account in Settings tab.', DKF_W1_TD ),
					),
					array(
						'title'  => __( 'Language', DKF_W1_TD ),
						'name'   => self::CULTURE_ID,
						'type'   => 'select',
						'values' => array(
							''      => __( 'Autodetection', DKF_W1_TD ),
							'ru-RU' => 'Русский',
							'en-US' => 'English',
						),
						'tip'    => __( 'Interface language is defined automatically, but you can choose to set it explicitly.', DKF_W1_TD ),
					),
					array(
						'title'  => __( 'Autolocation', DKF_W1_TD ),
						'name'   => self::AutoLocation,
						'type'   => 'checkbox',
						'desc'   => __( 'Filter By IP', DKF_W1_TD ),
						'value'  => 1,
						'tip'    => __( 'Allows to show the methods of payment available for user according to his location. "No Filter" — shows the methods regardless of the user\'s country; "Filter By IP" - the user\'s country and the display of methods determines according to the IP.', DKF_W1_TD ),
					),
				),
			),
		);

	}

	/**
	 * Retrieves the fields to be used in the payment form.
	 *
	 * @param APP_Order $order   An order to be processed.
	 * @param array     $options The gateway options.
	 *
	 * @return array The fields array.
	 */
	protected function get_form_fields( $order, $options ) {

		// Basic parameters.
		$fields = array(
			self::MERCHANT_ID    => $options[ self::MERCHANT_ID ],
			self::CULTURE_ID     => $options[ self::CULTURE_ID ],
			self::PAYMENT_AMOUNT => $order->get_total(),
			self::PAYMENT_NO     => $order->get_id(),
			self::SUCCESS_URL    => $order->get_return_url(),
			self::FAIL_URL       => $order->get_cancel_url(),
			self::CURRENCY_ID    => DKF_WalletOne_Currencies::get_num_code( $order->get_currency() ),
			self::DESCRIPTION    => "BASE64:" . base64_encode( $order->get_description() ),
			'WMI_CMS'            => 'Wordpress AppThemes',
			'CMS'                => 'Wordpress AppThemes',
		);

		$fields[ self::AutoLocation ] = ( $options[ self::AutoLocation ] ) ? 1 : 0;

		// Allow to change fields.
		return apply_filters( 'dkf_wallet_one_form_args', $fields );
	}

	/**
	 * Generates payment request form.
	 *
	 * @param APP_Order $order   An order to be processed.
	 * @param array     $options The gateway options.
	 */
	protected function create_form( $order, $options ) {

		$form = array(
			'action'         => self::POST_URL,
			'accept-charset' => 'utf-8',
			'id'             => 'create-listing',
			'method'         => 'POST',
			'name'           => 'wallet_one_payform',
		);

		// Get form fields.
		$fields = $this->get_form_fields( $order, $options );
		$key    = $options[ self::SECRET_KEY ];

		// Generate signature.
		$fields[ self::SIGNATURE ] = $this->_create_signature( $fields, $key );

		// Print form and redirect to checkout.
		$this->redirect( $form, $fields, __( 'You are now being redirected to Wallet One.', DKF_W1_TD ) );
	}

	/**
	 * Checks the payment server Success response.
	 *
	 * @param APP_Order $order   An order to be processed.
	 * @param array     $options The gateway options.
	 *
	 * @return boolean Checked result.
	 */
	protected function is_valid( $order, $options ) {

		$order = appthemes_get_order( $order->get_id() );

		if ( in_array( $order->get_status(), array( APPTHEMES_ORDER_COMPLETED, APPTHEMES_ORDER_ACTIVATED ) ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Process a recurring order.
	 *
	 * Wallet One processes recurring via IPN, therefore doesn't use this handler.
	 *
	 * @param APP_Order $order   The order to be processed.
	 * @param array     $options An array of user-entered options
	 * 				             corresponding to the values provided in form().
	 */
	public function process_recurring( $order, array $options ) {
		// API is not provided yet.
	}

	/**
	 * Verifies recieved data from Wallet One.
	 *
	 * @param array $data Posted data.
	 */
	public function verify_ipn_data( $data) {

		$order = appthemes_get_order( $data[ self::PAYMENT_NO ] );

		foreach ( array( self::SIGNATURE, self::ORDER_STATE ) as $arg ) {
			if ( ! isset( $data[ $arg ] ) ) {
				$message = sprintf( __( 'Missed parameter %s', DKF_W1_TD ), $arg );
				$order->log( $message );
				$this->print_answer( 'RETRY', $message );
			}
		}

		$posted_sign = $data[ self::SIGNATURE ];
		$order_state = $data[ self::ORDER_STATE ];

		unset( $data[ self::SIGNATURE ] );

		$options   = APP_Gateway_Registry::get_gateway_options( $this->identifier() );
		$signature = $this->_create_signature( $data, $options[ self::SECRET_KEY ] );

		if ( $signature === $posted_sign ) {

			if ( strtoupper( $order_state ) === 'ACCEPTED' ) {
				$order->complete();
				$this->print_answer( 'OK', sprintf( __( 'Order #%d has been completed.', DKF_W1_TD ), $order->get_id() ) );

			} else {
				$order->failed();
				$message = sprintf( __( 'Incorrect order state "%s".', DKF_W1_TD ), $order_state );
				$order->log( $message );
				$this->print_answer( 'RETRY', $message );
			}
		} else {
			$order->failed();
			$message = __( 'Ivalid signature.', DKF_W1_TD );
			$order->log( $message );
			$this->print_answer( 'RETRY', $message );
		}
	}

	/**
	 * Generates signature string from the given arguments.
	 *
	 * @param array  $fields Associative array of the form fields.
	 * @param string $key    Secret key.
	 *
	 * @return string Generated signature string.
	 */
	private function _create_signature( $fields, $key ) {

		foreach ( $fields as $name => $val ) {
			if ( is_array( $val ) ) {
				usort( $val, 'strcasecmp' );
				$fields[ $name ] = $val;
			}
		}

		uksort( $fields, 'strcasecmp' );

		$fieldValues = '';
		$charset = strtoupper( get_bloginfo( 'charset' ) );
		$charset = ( 'WINDOWS-1251' !== $charset ) ? get_bloginfo( 'charset' ) : '';

		foreach ( $fields as $value ) {
			if ( is_array( $value ) ) {
				foreach ( $value as $v ) {
					$fieldValues .= ( $charset ) ? iconv( $charset, 'windows-1251', $v ) : $v;
				}
			} else {
				$fieldValues .= ( $charset ) ? iconv( $charset, 'windows-1251', $value ) : $value;
			}
		}

		return base64_encode( pack( 'H*', md5( $fieldValues . $key ) ) );
	}

	/**
	 * Prints IPN request answer.
	 *
	 * @param string $result      The result code of request processing.
	 * @param string $description The result description.
	 */
	public function print_answer( $result, $description ) {
		print 'WMI_RESULT=' . esc_html( strtoupper( $result ) ) . '&';
		print 'WMI_DESCRIPTION=' .urlencode( $description );
		exit();
	}

}

/**
 * IPN Request view.
 */
class DKF_WalletOne_Order_View extends APP_View {

	/**
	 * The condition.
	 * @return type
	 */
	public function condition() {

		if ( ! isset( $_REQUEST[ DKF_WalletOne::ID ] ) || 'result' !== $_REQUEST[ DKF_WalletOne::ID ] ) {
			return false;
		}

		if ( isset( $_POST[ DKF_WalletOne::PAYMENT_NO ] ) ) {
			return true;
		}
	}

	/**
	 * Redirects to appropriate order url depending on GET parameter.
	 */
	public function template_redirect() {

		$posted  = stripslashes_deep( $_POST );
		$gateway = APP_Gateway_Registry::get_gateway( DKF_WalletOne::ID );

		if ( ! $gateway ) {
			return;
		}

		$gateway->verify_ipn_data( $posted );
		exit();

	}
}

new DKF_WalletOne_Order_View();
